Scriptname TweakLevelup extends ReferenceAlias

ReferenceAlias Property PlayerRef  Auto  

Int property previousLevel auto
Faction property CurrentFollowerFaction auto
Faction property TweakDisableMagicFaction auto
Faction property WerewolfFaction auto

Faction property pTweakManageLevelUpFaction auto

Int property actorType auto
TweakFollowerScript property pTweakFollowerScript auto

; The originalXXX variables were repurposed in 1.41 to hold the
; CURRENT value. Which only comes into play if the player 
; decides to manage the stats. 

int property originalHealth auto
int property originalMagicka auto
int property originalStamina auto
int property originalOneHanded auto
int property originalTwoHanded auto
int property originalMarksman auto
int property originalHeavy auto
int property originalLight auto
int property originalBlock auto
int property originalSneak auto
int property originalPick auto
int property originalLock auto
int property originalSmithing auto
int property originalSpeechcraft auto
int property originalAlchemy auto
int property originalConjuration auto
int property originalAlteration auto
int property originalEnchanting auto
int property originalDestruction auto
int property originalIllusion auto
int property originalRestoration auto
int property command auto
 
Bool property manual auto

Potion property pTweakCurrHealth Auto
Potion property pTweakCurrStamina Auto
Potion property pTweakCurrMagicka Auto
Book property pTweakCurrConjuration Auto
Book property pTweakCurrAlteration Auto
Book property pTweakCurrDestruction Auto
Book property pTweakCurrIllusion Auto
Book property pTweakCurrRestoration Auto
MiscObject property pTweakCurrAlchemy Auto
MiscObject property pTweakCurrSpeech Auto
MiscObject property pTweakCurrPickpocket Auto
MiscObject property pTweakCurrLockpick Auto
MiscObject property pTweakCurrSmithing Auto
MiscObject property pTweakCurrMarksman Auto
MiscObject property pTweakCurrSneak Auto
MiscObject property pTweakCurrOneHanded Auto
MiscObject property pTweakCurrTwoHanded Auto
MiscObject property pTweakCurrHeavy Auto
MiscObject property pTweakCurrLight Auto
MiscObject property pTweakCurrBlock Auto
MiscObject property pTweakCurrEnchanting Auto

MiscObject property pTweakLUSpacer Auto

Potion property pTweakRemainingStats Auto
MiscObject property pTweakRemainingSkills Auto
MiscObject property pTweakResetSkills Auto
Potion property pTweakResetStats Auto

float property version auto

Formlist property pTweakLevelUpItems Auto
TweakDawnGuardScript Property pTweakDawnGuardScript Auto

Function AftReset()

	; Skyrim levels NPCs as of version 1.6.8. So original doesnt matter anymore. All we have to
	; do is call setrace as it will re-calculate their stats to what they should be. This is already done
	; by tweakvampire. 

	initStats()
	previousLevel=0
	actorType=0
	manual = false
	command = 0
	
	self.GetActorRef().RemoveFromFaction(pTweakManageLevelUpFaction)

endFunction

Function initStats()
	originalHealth=50
	originalStamina=50
	originalMagicka=50
	originalOneHanded=15
	originalTwoHanded=15
	originalMarksman=15
	originalHeavy=15
	originalLight=15
	originalBlock=15
	originalSneak=15
	originalPick=15
	originalLock=15
	originalSmithing=15
	originalSpeechcraft=15
	originalAlchemy=15
	originalConjuration=15
	originalAlteration=15
	originalEnchanting=15
	originalDestruction=15
	originalIllusion=15
	originalRestoration=15
endFunction

Event OnUpdate()
	if (3 == command)
		command = 0
		ManualLevelUp()
	endif
endEvent

Event OnRaceSwitchComplete()
	Actor npc = self.GetActorRef()
	if !npc || npc.IsDead()
		return
	endIf

	Utility.wait(1.0)
	LevelUp(true)
	
endEvent

Function SetManual(Bool b)
	manual = b
	if (b)
		self.GetActorRef().AddToFaction(pTweakManageLevelUpFaction)
		command = 3
		RegisterForSingleUpdate(0.1)
	else
		self.GetActorRef().RemoveFromFaction(pTweakManageLevelUpFaction)
	endif
endFunction

Function ManualLevelUp()

	TweakPlayerRef pTweakPlayerRef = (PlayerREF As TweakPlayerRef)
	if (!pTweakPlayerRef)
		; Hook up script handler to pc
		PlayerREF.Clear()
		PlayerREF.ForceRefTo(Game.GetPlayer())
		Utility.wait(0.25)
		pTweakPlayerRef = (PlayerREF As TweakPlayerRef)
	EndIf

	TweakInventoryControl pTweakInventoryControl = (self As ReferenceAlias) As TweakInventoryControl
	
	if (!pTweakPlayerRef || !pTweakInventoryControl)
		Debug.Notification("Manual Levelup not available at this time")
		Return
	EndIf

	Actor npc = self.GetActorRef()

	if (npc.IsInFaction(WerewolfFaction))
		Debug.MessageBox("tH[ϐgĂԂł͂ł܂")
		return
	endif

	if (pTweakDawnGuardScript.installed)
		if npc.IsInFaction(pTweakDawnGuardScript.VampireLordFaction)
			Debug.MessageBox("tH[ϐgĂԂł͂ł܂")
			return
		endif
	endif
	
	Initv141()
	; compute available stats:
	int pcLevel    = (Game.GetPlayer().GetLevel() as Int)
	int currStats = originalHealth + originalStamina + originalMagicka
	int maxStats   = 0
	if (pTweakFollowerScript.followerNoBoostStats)
		maxStats = 300 + (10 * pcLevel)	
	else
		maxStats = 500 + (13 * pcLevel)	
	endif
	int availStats = (maxStats - currStats)
	if availStats < 0
		availStats = 0
	endIf
	int currSkills = (originalOneHanded + originalTwoHanded + originalMarksman)
	currSkills += (originalHeavy + originalLight + originalBlock)
	currSkills += (originalSneak + originalPick + originalLock)
	currSkills += (originalSmithing + originalSpeechcraft + originalAlchemy)
	currSkills += (originalConjuration + originalAlteration + originalEnchanting)
	currSkills += (originalDestruction + originalIllusion + originalRestoration)
	int maxSkills  = 270 + (7 * pcLevel)
	int availSkills = (maxSkills - currSkills)
	if availSkills < 0
		availSkills = 0
	endIf
	
	; Disable Inventory Reaction Events
	pTweakInventoryControl.tranSell = true 
	ObjectReference oref = self.GetRef()

	oref.AddItem(pTweakCurrHealth,originalHealth, true)
	oref.AddItem(pTweakCurrMagicka,originalMagicka, true)
	oref.AddItem(pTweakCurrStamina,originalStamina, true)
	oref.AddItem(pTweakCurrOneHanded,originalOneHanded, true)
	oref.AddItem(pTweakCurrTwoHanded,originalTwoHanded, true)
	oref.AddItem(pTweakCurrMarksman,originalMarksman, true)
	oref.AddItem(pTweakCurrHeavy,originalHeavy, true)
	oref.AddItem(pTweakCurrLight,originalLight, true)
	oref.AddItem(pTweakCurrBlock,originalBlock, true)
	oref.AddItem(pTweakCurrSneak,originalSneak, true)
	oref.AddItem(pTweakCurrPickpocket, originalPick, true)
	oref.AddItem(pTweakCurrLockpick, originalLock, true)
	oref.AddItem(pTweakCurrSmithing, originalSmithing, true)
	oref.AddItem(pTweakCurrSpeech, originalSpeechcraft, true)
	oref.AddItem(pTweakCurrAlchemy, originalAlchemy, true)
	oref.AddItem(pTweakCurrConjuration, originalConjuration, true)
	oref.AddItem(pTweakCurrAlteration, originalAlteration, true)
	oref.AddItem(pTweakCurrEnchanting, originalEnchanting, true)
	oref.AddItem(pTweakCurrDestruction, originalDestruction, true)
	oref.AddItem(pTweakCurrIllusion, originalIllusion, true)
	oref.AddItem(pTweakCurrRestoration, originalRestoration, true)

	oref.AddItem(pTweakRemainingStats, availStats, true)
	oref.AddItem(pTweakRemainingSkills, availSkills, true)

	oref.AddItem(pTweakLUSpacer, 1, true)
	oref.AddItem(pTweakResetSkills, 1, true)
	oref.AddItem(pTweakResetStats, 1, true)
	
	pTweakPlayerRef.StartLevelUp()

	npc.ShowGiftMenu(false, pTweakLevelUpItems, false, false)
	Utility.wait(0.01)
	pTweakPlayerRef.StopLevelUp()

	originalHealth=oref.GetItemCount(pTweakCurrHealth)
	originalMagicka=oref.GetItemCount(pTweakCurrMagicka)
	originalStamina=oref.GetItemCount(pTweakCurrStamina)
	originalOneHanded=oref.GetItemCount(pTweakCurrOneHanded)
	originalTwoHanded=oref.GetItemCount(pTweakCurrTwoHanded)
	originalMarksman=oref.GetItemCount(pTweakCurrMarksman)
	originalHeavy=oref.GetItemCount(pTweakCurrHeavy)
	originalLight=oref.GetItemCount(pTweakCurrLight)
	originalBlock=oref.GetItemCount(pTweakCurrBlock)
	originalSneak=oref.GetItemCount(pTweakCurrSneak)
	originalPick=oref.GetItemCount(pTweakCurrPickpocket)
	originalLock=oref.GetItemCount(pTweakCurrLockpick)
	originalSmithing=oref.GetItemCount(pTweakCurrSmithing)
	originalSpeechcraft=oref.GetItemCount(pTweakCurrSpeech)
	originalAlchemy=oref.GetItemCount(pTweakCurrAlchemy)
	originalConjuration=oref.GetItemCount(pTweakCurrConjuration)
	originalAlteration=oref.GetItemCount(pTweakCurrAlteration)
	originalEnchanting=oref.GetItemCount(pTweakCurrEnchanting)
	originalDestruction=oref.GetItemCount(pTweakCurrDestruction)
	originalIllusion=oref.GetItemCount(pTweakCurrIllusion)
	originalRestoration=oref.GetItemCount(pTweakCurrRestoration)

	Cleanup(oref)
	Cleanup(Game.GetPlayer())
	LevelUp(true)

	; Enable Inventory Reaction Events
	pTweakInventoryControl.tranSell = false

EndFunction

Function Cleanup(ObjectReference r)
	r.RemoveItem(pTweakCurrHealth,9999, true)
	r.RemoveItem(pTweakCurrMagicka,9999, true)
	r.RemoveItem(pTweakCurrStamina,9999, true)
	r.RemoveItem(pTweakCurrOneHanded,999, true)
	r.RemoveItem(pTweakCurrTwoHanded,999, true)
	r.RemoveItem(pTweakCurrMarksman,999, true)
	r.RemoveItem(pTweakCurrHeavy,999, true)
	r.RemoveItem(pTweakCurrLight,999, true)
	r.RemoveItem(pTweakCurrBlock,999, true)
	r.RemoveItem(pTweakCurrSneak,999, true)
	r.RemoveItem(pTweakCurrPickpocket, 999, true)
	r.RemoveItem(pTweakCurrLockpick, 999, true)
	r.RemoveItem(pTweakCurrSmithing, 999, true)
	r.RemoveItem(pTweakCurrSpeech, 999, true)
	r.RemoveItem(pTweakCurrAlchemy, 999, true)
	r.RemoveItem(pTweakCurrConjuration, 999, true)
	r.RemoveItem(pTweakCurrAlteration, 999, true)
	r.RemoveItem(pTweakCurrEnchanting, 999, true)
	r.RemoveItem(pTweakCurrDestruction, 999, true)
	r.RemoveItem(pTweakCurrIllusion, 999, true)
	r.RemoveItem(pTweakCurrRestoration, 999, true)

	r.RemoveItem(pTweakRemainingStats, 9999, true)
	r.RemoveItem(pTweakRemainingSkills, 9999, true)
	r.RemoveItem(pTweakLUSpacer, 99, true)
	r.RemoveItem(pTweakResetSkills, 99, true)
	r.RemoveItem(pTweakResetStats, 99, true)
endFunction

; Deprecated : 1.41
Int property defaultLevelUp auto

; Deprecated : 1.41
; This isn't needed anymore as we simply dont do anything if manual is false and they haven't chosen
; a combat style 
;
; 731 = Mage : Destruction, Alteration, Restoration, Conjuration, Sneak, Archery
; 732 = Spell Sword : 1 handed, Destruction, light armor,  Alteration, Conjuration, Restoration, Sneak
; 733 = 1 Handed : 1 handed, Block, heavy armor, light armor,  Archery, Sneak
; 734 = 2 handed : 2 handed, light armor, heavy armor, Block, Archery, Sneak
; 735 = Dual : 1 handed, light armor, heavy armor, Block, Archery, Sneak
; 736 = Bow : Archery, 1 handed, light armor, heavy armor, Block, Sneak
Function determineLevelUp()
endFunction

; One time operations (typically time consuming)
Function initialize()
	initStats()
	actorType=0
	manual = false
	version=1.41
	command=0
	previousLevel=self.GetActorRef().GetLevel()
endFunction

Function Initv141()
	if (version == 1.41)
		return
	endif
	version = 1.41
	initStats()
endFunction

; Every time they begin following the PC when previously not following the pc at all. 
Function EventFollowingPlayer()
	Bool happened = LevelUp()
	if (happened)
		if (pTweakFollowerScript.showMessages)
			String npcName = ((self As ReferenceAlias) As TweakSettings).originalName
			Debug.MessageBox(npcName + "xAbv!")		
		endif
	endif
endFunction

; Every time they continue following the PC after waiting
Function EventWaitToFollow()
	; Nothing to do right now.
EndFunction

; When they stop following the PC
Function EventNotFollowingPlayer()
	; Nothing to do right now.
endFunction

; Event comes from Quest TweakLevelMonitor which gets event from Story Manager Event Node TweakLMKickNode. It
; bumps this quest which relays event to all current followers. Note this doesnt fire if user uses console player.setlevel 
; to level up. 

Function OnPCLevelUp()
	Actor npc = self.GetActorRef()
	if !npc
		return
	endIf
	if (!npc.IsInFaction(CurrentFollowerFaction))
	
		; Bug Report - Dismissed NPCs become naked if in same cell as Player when Player Levels up
		; Specifically this was seen during a dawnguard battle. Unable to reproduce. Left commented
		; out as I dont want players blinking if I dont have to

		;if ( self.GetReference().Is3DLoaded() )
		;	TweakInventoryControl pTweakInventoryControl = ( self As ReferenceAlias ) As TweakInventoryControl
		;	if (pTweakInventoryControl)
		;		pTweakInventoryControl.OnCellLoad()
		;	endif
		;endif

		return
	endif
	Bool happened = LevelUp()
	if (happened)
		if (pTweakFollowerScript.showMessages)
			String npcName = ((self As ReferenceAlias) As TweakSettings).originalName
			Debug.MessageBox(npcName + "xAbv!")		
		endif
	endif
EndFunction

; Called from werewolf Deprecated 1.41. Now just relays to Level Up
Function ResetAndLevelUp()
	LevelUp(true)
EndFunction

; Gets called under 3 scenarios
; 1 PC levels up
; 2 Race Change Event (When werewolf to human, Force will be true) 
; 3 Follower is asked to follow after having been dismissed for some time
;
; Notes: Some NPCs are set to level up faster than the PC. Mjoll will always be twice the
; level of the PC for example. Therefore, we should NEVER pay attention to the NPCs level. 
; Always base things on the PCs level. 

Bool Function LevelUp(Bool force=false)

	Actor npc = self.GetActorRef()

	; We check for faction instead of race incase user has installed mod that adds custom werewolf race.
	; We would still be compatible as long as they are properly added to the werewolf faction.

	if (npc.IsInFaction(WerewolfFaction))
		npc.SetAv("Magicka", 0)
		int curMagicka = npc.GetAv("Magicka") as Int
		if (curMagicka != 0)
			npc.ModAv("Magicka", (0 - curMagicka))
		endif
		TweakMagic pTweakMagic = ((self as ReferenceAlias) As TweakMagic)
		pTweakMagic.magicRestore = 0
		return false
	endif

	int combatType = self.GetActorRef().GetAV("FavorsPerDayTimer") As Int
	int npcLevel = npc.GetLevel() as Int

	if (!manual && combatType < 731)
		; Let the game deal with it. 
		TweakMagic pTweakMagic = ((self as ReferenceAlias) As TweakMagic)
		pTweakMagic.magicRestore = (npc.GetAv("Magicka") as Int)
		if (npcLevel > previousLevel)
			previousLevel = npcLevel
			return true
		endif
		return false
	endif
	
	npcLevel = (Game.GetPlayer().GetLevel() as Int)

	; Do we need to Level the NPC Up?
	if (npcLevel > previousLevel || force)

		if (npcLevel > previousLevel)
			previousLevel = npcLevel
		endif

		int minHealth=150
		int minMagicka=150
		int minStamina=200
		int minOneHanded=15
		int minTwoHanded=15
		int minMarksman=15
		int minHeavy=15
		int minLight=15
		int minBlock=15
		int minSneak=15
		int minPick=15
		int minLock=15
		int minSmithing=15
		int minSpeechcraft=15		
		int minAlchemy=15
		int minConjuration=15
		int minAlteration=15
		int minEnchanting=15
		int minDestruction=15
		int minIllusion=15
		int minRestoration=15

		if (manual)

			minHealth = originalHealth
			minMagicka = originalMagicka
			minStamina = originalStamina
			minOneHanded = originalOneHanded
			minTwoHanded = originalTwoHanded
			minMarksman = originalMarksman
			minHeavy = originalHeavy
			minLight = originalLight
			minBlock = originalBlock
			minSneak = originalSneak
			minPick = originalPick
			minLock = originalLock
			minSmithing=originalSmithing
			minSpeechcraft=originalSpeechcraft
			minAlchemy = originalAlchemy
			minConjuration = originalConjuration
			minAlteration = originalAlteration
			minEnchanting = originalEnchanting
			minDestruction = originalDestruction
			minIllusion = originalIllusion
			minRestoration = originalRestoration

		else ; AUTO

			;Calculate what the skill values SHOULD be and ensure Minimums:

			if (pTweakFollowerScript.followerNoBoostStats)
				if (731 == combatType || 732 == combatType) ; MAGE or SpellSword
					minHealth   = 100
					minMagicka = 100
					minStamina = 100
				else
					minHealth   = 125
					minMagicka = 50
					minStamina = 125
				endif
			endif

			; Everyone is designed to max out when pc reaches level 75	
			if (731 == combatType) ; MAGE
				if (pTweakFollowerScript.followerNoBoostStats)
					minMagicka += (npcLevel * 5)
					minHealth += (npcLevel * 3)
					minStamina += (npcLevel * 2)
				else
					minMagicka += (npcLevel * 6)
					minHealth += (npcLevel * 4)
					minStamina += (npcLevel * 3)
				endif
				minDestruction += (15 + npcLevel)
				minConjuration += (15 + npcLevel)
				minRestoration += (15 + npcLevel)
				minAlteration += (15 + npcLevel)
				minSneak += (5 + npcLevel)
				minOneHanded += ((npcLevel/2) as Int)
				minLight += ((npcLevel/2) as Int)
				minBlock += ((npcLevel/2) as Int)

			elseif (732 == combatType) ; SPELLSWORD

				if (pTweakFollowerScript.followerNoBoostStats)
					minMagicka += (npcLevel * 4)
					minHealth += (npcLevel * 3)
					minStamina += (npcLevel * 3)
				else
					minMagicka += (npcLevel * 4)
					minHealth += (npcLevel * 5)
					minStamina += (npcLevel * 4)

					; Compensate for lack of perk support
					if (npcLevel > 50)
						minMagicka = minMagicka * 2
					endif
				
				endif
				minOneHanded += (15 + npcLevel)
				minDestruction += (15 + npcLevel)
				minLight += (10 + npcLevel)
				minAlteration += (10 + npcLevel)
				minConjuration += (10 + npcLevel)
				minRestoration += (5 + npcLevel)
				minSneak += (5 + npcLevel)

			elseif (734 == combatType) ; TwoHanded
				if (pTweakFollowerScript.followerNoBoostStats)
					minHealth += (npcLevel * 5)
					minStamina += (npcLevel * 5)
				else
					minMagicka += (npcLevel * 2)
					minHealth += (npcLevel * 7)
					minStamina += (npcLevel * 5)
				endif
				minTwoHanded += (15 + npcLevel)
				minBlock += (15 + npcLevel)
				minLight += (15 + npcLevel)
				minHeavy += (15 + npcLevel)
				minMarksman += (5 + npcLevel)
				minSneak += (5 + npcLevel)
			elseif (735 == combatType) ; Dual
				if (pTweakFollowerScript.followerNoBoostStats)
					minHealth += (npcLevel * 5)
					minStamina += (npcLevel * 5)
				else
					minMagicka += (npcLevel * 2)
					minHealth += (npcLevel * 7)
					minStamina += (npcLevel * 5)
				endif
				minOneHanded += (15 + npcLevel)
				minLight += (15 + npcLevel)
				minLock += (10 + npcLevel)
				minSneak += (10 + npcLevel)
				minPick += (10 + npcLevel)
				minAlchemy += (5 + npcLevel)
				minMarksman += (5 + npcLevel)
			elseif (736 == combatType) ; Bow
				if (pTweakFollowerScript.followerNoBoostStats)
					minHealth += (npcLevel * 5)
					minStamina += (npcLevel * 5)
				else
					minMagicka += (npcLevel * 2)
					minHealth += (npcLevel * 7)
					minStamina += (npcLevel * 5)
				endif
				minMarksman += (15 + npcLevel)
				minLight += (15 + npcLevel)
				minLock += (10 + npcLevel)
				minSneak += (10 + npcLevel)
				minPick += (10 + npcLevel)
				minAlchemy += (5 + npcLevel)
				minOneHanded += (5 + npcLevel)
			elseif (737 == combatType) ; Ranger
				if (pTweakFollowerScript.followerNoBoostStats)
					minHealth += (npcLevel * 5)
					minStamina += (npcLevel * 5)
				else
					minMagicka += (npcLevel * 2)
					minHealth += (npcLevel * 7)
					minStamina += (npcLevel * 5)
				endif
				minMarksman += (15 + npcLevel)
				minLight += (15 + npcLevel)
				minLock += (5 + npcLevel)
				minSneak += (10 + npcLevel)
				minPick += (5 + npcLevel)
				minAlchemy += (5 + npcLevel)
				minOneHanded += (15 + npcLevel)
			elseif (738 == combatType) ; General
				if (pTweakFollowerScript.followerNoBoostStats)
					minHealth += (npcLevel * 5)
					minStamina += (npcLevel * 5)
				else
					minMagicka += (npcLevel * 4)
					minHealth += (npcLevel * 5)
					minStamina += (npcLevel * 5)
				endif
				minOneHanded += (10 + npcLevel)
				minTwoHanded += (10 + npcLevel)
				minLight += (10 + npcLevel)
				minHeavy += (10 + npcLevel)
				minMarksman += (10 + npcLevel)
				minSneak += (10 + npcLevel)
				minDestruction += (10 + npcLevel)
			else ; Assume 733 1Handed
				if (pTweakFollowerScript.followerNoBoostStats)
					minHealth += (npcLevel * 5)
					minStamina += (npcLevel * 5)
				else
					minMagicka += (npcLevel * 2)
					minHealth += (npcLevel * 7)
					minStamina += (npcLevel * 5)
				endif
				minOneHanded += (15 + npcLevel)
				minBlock += (15 + npcLevel)
				minLight += (10 + npcLevel)
				minHeavy += (10 + npcLevel)
				minMarksman += (5 + npcLevel)
				minSneak += (5 + npcLevel)
			endif

			if (minOneHanded > 100)
				minOneHanded = 100
			endif
			if (minTwoHanded > 100)
				minTwoHanded = 100
			endif
			if (minMarksman > 100)
				minMarksman = 100
			endif
			if (minBlock > 100)
				minBlock = 100
			endif
			if (minHeavy > 100)
				minHeavy = 100
			endif
			if (minLight > 100)
				minLight = 100
			endif
			if (minPick > 100)
				minPick = 100
			endif
			if (minLock > 100)
				minLock = 100
			endif
			if (minSneak > 100)
				minSneak = 100
			endif
			if (minAlchemy > 100)
				minAlchemy = 100
			endif
			if (minConjuration > 100)
				minConjuration = 100
			endif
			if (minDestruction > 100)
				minDestruction = 100
			endif
			if (minRestoration > 100)
				minRestoration = 100
			endif
			if (minAlteration > 100)
				minAlteration = 100
			endif
		endif

		; Debug.Notification("Level-Up: Setting Health")

		npc.SetAv("Health", minHealth)
		npc.SetAv("Stamina", minStamina)
		npc.SetAv("OneHanded", minOneHanded)
		npc.SetAv("TwoHanded", minTwoHanded)
		npc.SetAv("Marksman", minMarksman)
		npc.SetAv("HeavyArmor", minHeavy)
		npc.SetAv("LightArmor", minLight)
		npc.SetAv("Block", minBlock)
		npc.SetAv("Sneak", minSneak)
		npc.SetAv("PickPocket", minPick)
		npc.SetAv("LockPicking", minLock)
		npc.SetAv("Smithing", minSmithing)
		npc.SetAv("Speechcraft", minSpeechcraft)
		npc.SetAv("Alchemy", minAlchemy)
		npc.SetAv("Conjuration", minConjuration)
		npc.SetAv("Alteration", minAlteration)
		npc.SetAv("Enchanting", minEnchanting)
		npc.SetAv("Destruction", minDestruction)
		npc.SetAv("Illusion", minIllusion)
		npc.SetAv("Restoration", minRestoration)

		int curHealth = (npc.GetAv("Health") as Int)
		int curStamina = (npc.GetAv("Stamina") as Int)
		int curOneHanded = (npc.GetAv("OneHanded") as Int)
		int curTwoHanded = (npc.GetAv("TwoHanded") as Int)
		int curMarksman = (npc.GetAv("Marksman") as Int)
		int curHeavy = (npc.GetAv("HeavyArmor") as Int)
		int curLight = (npc.GetAv("LightArmor") as Int)
		int curBlock = (npc.GetAv("Block") as Int)
		int curSneak = (npc.GetAv("Sneak") as Int)
		int curPick = (npc.GetAv("PickPocket") as Int)
		int curLock = (npc.GetAv("LockPicking") as Int)
		int curSmithing = (npc.GetAv("Smithing") as Int)
		int curSpeechcraft = (npc.GetAv("Speechcraft") as Int)
		int curAlchemy = (npc.GetAv("Alchemy") as Int)
		int curConjuration = (npc.GetAv("Conjuration") as Int)
		int curAlteration = (npc.GetAv("Alteration") as Int)
		int curEnchanting = (npc.GetAv("Enchanting") as Int)
		int curDestruction = (npc.GetAv("Destruction") as Int)
		int curIllusion = (npc.GetAv("Illusion") as Int)
		int curRestoration = (npc.GetAv("Restoration") as Int)

		if (curHealth < minHealth)
			npc.ModAv("Health", (minHealth - curHealth))
		endif
		if (curStamina < minStamina)
			npc.ModAv("Stamina", (minStamina - curStamina))
		endif
		if (curOneHanded < minOneHanded)
			npc.ModAv("OneHanded", (minOneHanded - curOneHanded))
		endif
		if (curTwoHanded < minTwoHanded)
			npc.ModAv("TwoHanded", (minTwoHanded - curTwoHanded))
		endif
		if (curMarksman < minMarksman)
			npc.ModAv("Marksman", (minMarksman - curMarksman))
		endif
		if (curHeavy < minHeavy)
			npc.ModAv("HeavyArmor", (minHeavy - curHeavy))
		endif
		if (curLight < minLight)
			npc.ModAv("LightArmor", (minLight - curLight))
		endif
		if (curBlock < minBlock)
			npc.ModAv("Block", (minBlock - curBlock))
		endif
		if (curSneak < minSneak)
			npc.ModAv("Sneak", (minSneak - curSneak))
		endif
		if (curPick < minPick)
			npc.ModAv("PickPocket", (minPick - curPick))
		endif
		if (curLock < minLock)
			npc.ModAv("LockPicking", (minLock - curLock))
		endif
		if (curSmithing < minSmithing)
			npc.ModAv("Smithing", (minSmithing - curSmithing))
		endif
		if (curSpeechcraft < minSpeechcraft)
			npc.ModAv("Speechcraft", (minSpeechcraft - curSpeechcraft))
		endif
		if (curAlchemy < minAlchemy)
			npc.ModAv("Alchemy", (minAlchemy - curAlchemy))
		endif
		if (curConjuration < minConjuration)
			npc.ModAv("Conjuration", (minConjuration - curConjuration))
		endif
		if (curAlteration < minAlteration)
			npc.ModAv("Alteration", (minAlteration - curAlteration))
		endif
		if (curEnchanting < minEnchanting)
			npc.ModAv("Enchanting", (minEnchanting - curEnchanting))
		endif
		if (curDestruction < minDestruction)
			npc.ModAv("Destruction", (minDestruction - curDestruction))
		endif
		if (curIllusion < minIllusion)
			npc.ModAv("Illusion", (minIllusion - curIllusion))
		endif
		if (curRestoration < minRestoration)
			npc.ModAv("Restoration", (minRestoration - curRestoration))
		endif

		if (npc.IsInFaction(TweakDisableMagicFaction))
			npc.SetAv("Magicka", 0)
			int curMagicka = npc.GetAv("Magicka") as Int
			if (curMagicka != 0)
				npc.ModAv("Magicka", (0 - curMagicka))
			endif
			TweakMagic pTweakMagic = ((self as ReferenceAlias) As TweakMagic)
			pTweakMagic.magicRestore = 0
		else

			if (!pTweakFollowerScript.followerNoBoostStats)
				; Compensate for lack of perk support
				if (731 == combatType) ; MAGE
					; Compensate for lack of perk support
					if (npcLevel < 60)
						if (npcLevel > 30)
							minMagicka = minMagicka * 2
						endif
					else
						minMagicka = minMagicka * 3
					endif
				elseif (732 == combatType) ; SPELLSWORD
					if (npcLevel > 50)
						minMagicka = minMagicka * 2
					endif
				endif
			endif

			npc.SetAv("Magicka", minMagicka)
			int curMagicka = npc.GetAv("Magicka") as Int
			if (curMagicka < minMagicka)
				npc.ModAv("Magicka", (minMagicka - curMagicka))
			endif
			TweakMagic pTweakMagic = ((self as ReferenceAlias) As TweakMagic)
			pTweakMagic.magicRestore = minMagicka
		endif
		return true
	endif
	return false
endFunction

