Scriptname TweakStables extends Quest  

Faction property TweakCurrentHorseFaction auto
Faction property TweakPCHorseFaction auto
Faction property PlayerHorseFaction auto

ReferenceAlias Property MarkarthHorse  Auto  
ReferenceAlias Property WhiterunHorse  Auto  
ReferenceAlias Property WindhelmHorse  Auto  
ReferenceAlias Property RiftenHorse    Auto  
ReferenceAlias Property SolitudeHorse  Auto  

ReferenceAlias Property ExtraHorse Auto  
ObjectReference Property ExtraHorseStart     Auto  
ActorBase Property TweakFence Auto

ObjectReference Property MarkarthPlayersHorseStart  Auto  
ObjectReference Property WhiterunPlayersHorseStart  Auto  
ObjectReference Property WindhelmPlayersHorseStart  Auto  
ObjectReference Property RiftenPlayersHorseStart  Auto  
ObjectReference Property SolitudePlayersHorseStart  Auto  

ObjectReference Property playerHorse Auto

TweakFollowerScript property pTweakFollowerScript auto
TweakDFScript property pTweakDFScript auto

Bool property OnLoadFlag auto
Bool property OnUnLoadFlag auto
Bool property OnLocChangeFlag auto
Bool property roundUpInProgress auto

float Property iversion Auto
Faction property PlayerFaction auto
Race property pDogCompanionRace Auto

Event OnInit()
	iversion = 1.3
	playerHorse = None
	OnLoadFlag = false
	OnLocChangeFlag = false
	OnUnLoadFlag = false
	roundUpInProgress = false
endEvent

Function AftReset()

	UnregisterForUpdate()
	ReferenceAlias[] horses = GetAllTweakHorses()
	Int i = 0
	while (i != horses.length)
		actor horse = horses[i].GetActorRef()
		; CleanUp just removes them from factions so they return to their stables.
		CleanUpHorse(horse)
		(horses[i] As TweakHorseScript).AftReset()
		i = i + 1
	endWhile

	playerHorse = None
	OnLoadFlag = false
	OnLocChangeFlag = false
	OnUnLoadFlag = false
	roundUpInProgress = false

endFunction

; Relayed from Quest TweakMonitorPlayer
Function OnMountHorse(ObjectReference theHorse)

	; When mounting, Followers may be in beast form.
	; So we relay the event to TweakFollower so that 
	; it can tell followers to untransform before
	; updating the AI. 

	SetPlayerHorse(theHorse)

endFunction

; Relayed from Quest TweakMonitorPlayer
Function OnUnMountHorse(ObjectReference theHorse)
	Actor pcHorse = theHorse As Actor
	if (!pcHorse)
		return
	endif
	if (pcHorse.IsDead())	
		return
	endif
	if (pTweakFollowerScript)
		pTweakFollowerScript.SetRiding(false)
	endif
endFunction

Function SetPlayerHorse(ObjectReference theHorse)

	if (!theHorse)
		return
	endif
	Actor pcHorse = theHorse As Actor
	if (!pcHorse)
		return
	endif
	if (pcHorse.IsDead())	
		return
	endif

	if (pTweakFollowerScript)
		pTweakFollowerScript.SetRiding(true)
	endif

	; Is the player horse already assigned?
	if (playerHorse)
		if (playerHorse == theHorse && iversion == 1.3)
			return
		endIf
		if (iversion != 1.3)
			iversion = 1.3
		endif
		(playerHorse As Actor).RemoveFromFaction(TweakPCHorseFaction)
	endif
	
	playerHorse = theHorse
		
	if (!IsStandard(playerHorse))

		; ExtraHorse stores off ptr to non-standard horse. You may
		; 1 non-standard horse. 
		
		ExtraHorse.Clear()
		ExtraHorseStart.Delete()
		ExtraHorseStart = None

		ExtraHorse.ForceRefTo(theHorse)

		; fence is a placeholder invisible actorthat doesn't move
		ExtraHorseStart = theHorse.PlaceAtMe(TweakFence)
		(ExtraHorseStart  As Actor).SetUnconscious(true)
		(ExtraHorseStart  As Actor).AllowPCDialogue(false)

		if (None == theHorse.GetFactionOwner() && None == theHorse.GetActorOwner())
			if (pTweakFollowerScript.followerCanRide)
				; Enable Followers to ride the horse
				pcHorse.SetFactionRank(PlayerHorseFaction, 1)
				theHorse.SetFactionOwner(PlayerFaction)
			endif
		endif

	endif

	UpdateHorses()
	pcHorse.AddToFaction(TweakPCHorseFaction)
	
	SetHorsesProtected(true)
	if (!pcHorse.GetActorBase().IsEssential())
		pcHorse.GetActorBase().SetProtected(true)
	endif
	
	pcHorse.EvaluatePackage()

endFunction

; Set AI to all owned horses to TweakCurrentHorse so that they wait whereever they are
Function UpdateHorses()
	ReferenceAlias[] horses = GetAllTweakHorses()
	Int i = 0
	while (i != horses.length)
		UpdateHorse(horses[i].GetActorRef())
		i = i + 1
	endWhile
endFunction

Function UpdateHorse(ObjectReference horse)
	Actor theHorse = horse as Actor
	theHorse.AddToFaction(TweakCurrentHorseFaction)
	theHorse.RemoveFromFaction(TweakPCHorseFaction)
	theHorse.EvaluatePackage()
	Utility.wait(0.1)
endFunction

Function SetHorsesProtected(Bool Value)
	ReferenceAlias[] horses = GetAllTweakHorses()
	Int i = 0
	while (i != horses.length)
		; If you set an essential actor to proctected it clears their essential flag which could break things 
		if (!horses[i].GetActorRef().GetActorBase().IsEssential() && !horses[i].GetActorRef().IsDead())
			horses[i].GetActorRef().GetActorBase().SetProtected(Value)
		endif
		i = i + 1
	endWhile
endFunction

Function RoundUpHorses()

	;Debug.MessageBox(" RoundUpHorses")

	roundUpInProgress = true

	Actor moveTarget = Game.GetPlayer()
	ReferenceAlias[] horses = GetAllTweakHorses()
	if (0 == horses.length)
		if (pTweakFollowerScript.showMessages)
			Debug.MessageBox("Ȃ͔nLĂ܂")
		endif
		RegisterForSingleUpdateGameTime(0.05) ; about 7.5 seconds
		return
	endif

	int[] offset = new int[6]
	offset[0] = 0
	offset[1] = 45
	offset[2] = -45
	offset[3] = 90
	offset[4] = -90
	offset[5] = 135


	Int i = 0
	Location hloc = None
	Location mloc = None
	while (i != horses.length && i < 6)
		actor horse = horses[i].GetActorRef()
		if (!horse.IsInFaction(TweakPCHorseFaction))
			UpdateHorse(horse)
		endif
		float[] pos = TraceCircle( Game.GetPlayer(), 256, offset[i])
		hloc = horse.GetCurrentLocation()
		mloc = moveTarget.GetCurrentLocation()
		if (hloc && mloc && !hloc.IsSameLocation(mloc))
			horse.MoveTo(moveTarget)
		endif
		horse.SetPosition(pos[0],pos[1],pos[2])
		horse.SetAngle(Game.GetPlayer().GetAngleX(), Game.GetPlayer().GetAngleY(), Game.GetPlayer().GetAngleZ() - 180)
		moveTarget = horse
		i = i + 1
	endWhile
	RegisterForSingleUpdateGameTime(0.04) ; about 6 seconds

endFunction

Function UpgradeHorses()
	ReferenceAlias[] horses = GetAllTweakHorses()
	Int i = 0
	while (i != horses.length)
		actor horse = horses[i].GetActorRef()
		CleanUpHorse(horse)
		(horses[i] As TweakHorseScript).AftReset()
		i = i + 1
	endWhile
endFunction

Function StableHorses()

	;Debug.MessageBox("StableHorses")
	roundUpInProgress = false
	ReferenceAlias[] horses = GetAllTweakHorses()
	Int i = 0
	while (i != horses.length)
		actor horse = horses[i].GetActorRef()
		CleanUpHorse(horse)
		i = i + 1
	endWhile

endFunction

Function CleanUpHorse(ObjectReference theHorse)

	actor horse = theHorse As Actor
	if (horse)
		horse.RemoveFromFaction(TweakCurrentHorseFaction)
		horse.RemoveFromFaction(TweakPCHorseFaction)
		if (horse.IsDead())
			return
		endif
		if (WhiterunHorse.GetActorRef() && WhiterunHorse.GetActorRef() == theHorse)	
			horse.MoveTo(WhiterunPlayersHorseStart)
		elseif (WindhelmHorse.GetActorRef() && WindhelmHorse.GetActorRef() == theHorse)
			horse.MoveTo(WindhelmPlayersHorseStart)
		elseif (RiftenHorse.GetActorRef() && RiftenHorse.GetActorRef() == theHorse)
			horse.MoveTo(RiftenPlayersHorseStart)
		elseif (MarkarthHorse.GetActorRef() && MarkarthHorse.GetActorRef() == theHorse)
			horse.MoveTo(MarkarthPlayersHorseStart)
		elseif (SolitudeHorse.GetActorRef() && SolitudeHorse.GetActorRef() == theHorse)
			horse.MoveTo(SolitudePlayersHorseStart)
		elseif (ExtraHorse.GetActorRef() && ExtraHorse.GetActorRef() == theHorse)
			horse.MoveTo(ExtraHorseStart)
		endif
	endif

endFunction

Function OnLoadHelper(ObjectReference theHorse)
	if (roundUpInProgress)
		return
	endif
	OnLoadFlag = true
	UnregisterForUpdate()
	RegisterForSingleUpdate(0.3)
endFunction

Function OnLocationChangeHelper(ObjectReference theHorse)
	if (Game.GetPlayersLastRiddenHorse().GetPlayerControls())
		OnLocChangeFlag = true
		UnregisterForUpdate()
		RegisterForSingleUpdate(0.9)
	endif
endFunction

Function OnUnLoadHelper(ObjectReference theHorse)
	roundUpInProgress = false
	OnUnLoadFlag = true
	UnregisterForUpdate()
	RegisterForSingleUpdate(1.5)
endFunction



Event OnUpdateGameTime()

	if (roundUpInProgress)
		roundUpInProgress = false
	endif

endEvent

Event OnUpdate()

	if (!playerHorse)
		return
	endif

	if (OnLoadFlag)
		OnLoadFlag = false
		HandleOnLoad()
	elseif (OnLocChangeFlag)
		OnLocChangeFlag = false
		HandleLocationChange()
	elseif (OnUnLoadFlag)
		OnUnLoadFlag = false
		HandleOnUnload()
	endif

	OnUnLoadFlag = false
	OnLoadFlag = false
	OnLocChangeFlag = false

endEvent

Function HandleOnUnload()

	; Debug.MessageBox("Horse Script OnUnLoad")

	; onUnLoad will never fire when player is riding a horse. So, if we are getting this event it is because the
	; player has left the horses cell on foot. If OnLocationChange also fires, then the player fast travelled
	; to an outdoor area. If it does not fire, they did not fast travelled or they transitioned to an interior cell.
	; either way, we put the followers behind them: 

	pTweakFollowerScript.GetFollowers()

endFunction

Function HandleOnLoad()

	; Debug.MessageBox("Horse Script OnLoad. This normally doesnt happen when player is riding a horse.")

	; onLoad will never fire when player is riding a horse. So, if we are getting this event it is because the
	; horse travelled to meet PC at location. (No one is riding) Still need to teleport horses though...
	; Just need to skip the follower teleport

	if (!pTweakFollowerScript.followerCanRide)
		return
	endif

	ReferenceAlias[] allfollowers = pTweakFollowerScript.GetAllTweakFollowers(true, true)
	int totalfollowcount = 0
	totalfollowcount = allfollowers.length
	int neededHorses = 0

	if (!pDogCompanionRace)
		pDogCompanionRace = Game.GetForm(0x000F1AC4) As Race
	endif
	while (totalfollowcount > 0)
		totalfollowcount -= 1
		if (allfollowers[totalfollowcount].GetActorRef().GetRace() != pDogCompanionRace)
			neededHorses += 1
		endif
	endwhile
	if (0 == neededHorses)
		return
	endif

	; Debug.Notification("Need [" + neededHorses + "] horses")

	ReferenceAlias[] horses = GetAllTweakHorses(true, true)
	if (0 == horses.length)
		return
	endif

	int[] offset = new int[6]
	offset[0] = 180
	offset[1] = 135
	offset[2] = -135
	offset[3] = 90
	offset[4] = -90
	offset[5] = 45

	Actor pcHorse = playerHorse as Actor
	int bail = 10
	Utility.wait(0.01)
	While(FALSE == pcHorse.Is3DLoaded() && 0 != bail)
		Utility.wait(0.25)
		bail -= 1
	EndWhile
	Actor moveTarget = pcHorse
	if (neededHorses > 6)
		neededHorses = 6
	endif

	Int i = 0
	Location hloc = None
	Location mloc = None
	while (i != horses.length && i < neededHorses)
		actor horse = horses[i].GetActorRef()
		hloc = horse.GetCurrentLocation()
		mloc = moveTarget.GetCurrentLocation()
		if (hloc && mloc && !hloc.IsSameLocation(mloc))
			horse.MoveTo(moveTarget)
		endif
		float[] pos = TraceCircle(pcHorse, 256, offset[i])
		horse.SetPosition(pos[0],pos[1],pos[2])
		horse.SetAngle(pcHorse.GetAngleX(), pcHorse.GetAngleY(), pcHorse.GetAngleZ())
		moveTarget = horse
		i = i + 1
	endWhile

endFunction

Function HandleLocationChange()

;	Debug.Notification("Horse Script OnLocationChanged. We expect this to happen when player is riding horse.")

	Actor pcHorse = playerHorse as Actor
	if (!pcHorse)
		return
	endif

	if (!pTweakFollowerScript.followerCanRide)
		return
	endif

	Bool isRiding = (Game.GetPlayersLastRiddenHorse().GetPlayerControls())

	ReferenceAlias[] allfollowers = pTweakFollowerScript.GetAllTweakFollowers(true, true)
	ReferenceAlias[] humanfollowers = new ReferenceAlias[5]
	int totalFollowCount = allfollowers.length
	int neededHorses = 0
	
	if (!pDogCompanionRace)
		pDogCompanionRace = Game.GetForm(0x000F1AC4) As Race
	endif

	while (totalFollowCount > 0)
		totalFollowCount -= 1
		if (allfollowers[totalFollowCount].GetActorRef().GetRace() != pDogCompanionRace)
			humanfollowers[neededHorses] = allfollowers[totalFollowCount]
			neededHorses += 1
		endif
	endwhile
	if (0 == neededHorses)
		return
	endif
	if (neededHorses > 6)
		neededHorses = 6
	endif

	; Debug.Notification("Need [" + neededHorses + "] horses")

	ReferenceAlias[] horses
	horses = GetAllTweakHorses(true, true, true) ; get owned and living horses that are nearby that the pc has not claimed
	int numhorses = horses.length
	if (numhorses >= neededHorses)
		; Need to cleanup
		; get owned and living horses that are NOT nearby or claimed by PC
		horses = GetAllTweakHorses(true, true, false, true) 
		if (!horses)
			return
		endif	
		Int i = 0
		while (i != horses.length)
			CleanUpHorse(horses[i].GetActorRef())
			i = i + 1
		endWhile
		return
	else
		neededHorses = (neededHorses - numhorses)
	endif

	horses = GetAllTweakHorses(false, true, false, true) ; get owned and living horses that are NOT nearby regardless of who has claimed them
	if (0 == horses.length)
		;Debug.Notification("No additional horses available. Bailing")
		return
	endif

	; This is most likely to happen after fast travel

	int count = 0
	int bail = 10
	int[] offset = new int[6]
	offset[0] = 180
	offset[1] = 135
	offset[2] = -135
	offset[3] = 90
	offset[4] = -90
	offset[5] = 45

	Actor moveTarget = pcHorse

	Int i = 0
	Location hloc = None
	Location mloc = None
	while (i != horses.length && count < neededHorses)
		actor horse = horses[i].GetActorRef()
		if (!horse.IsInFaction(TweakPCHorseFaction))
			hloc = horse.GetCurrentLocation()
			mloc = moveTarget.GetCurrentLocation()
			if (hloc && mloc && !hloc.IsSameLocation(mloc))
				horse.MoveTo(moveTarget)
			endif
			float[] pos = TraceCircle(pcHorse, 256, offset[count])
			horse.SetPosition(pos[0],pos[1],pos[2])
			horse.SetAngle(pcHorse.GetAngleX(), pcHorse.GetAngleY(), pcHorse.GetAngleZ())
			Utility.wait(0.15)
			bail = 10
			While(FALSE == horse.Is3DLoaded() && 0 != bail)
				Utility.wait(0.20)
				bail -= 1
			EndWhile
			moveTarget = horse
			if (isRiding)
				Actor rider = humanFollowers[count].GetActorRef()
				utility.wait(0.001)
				rider.MoveTo(moveTarget)
				Utility.wait(0.01)
				bail = 10
				While(FALSE == rider.Is3DLoaded() && 0 != bail)
					Utility.wait(0.20)
					bail -= 1
				EndWhile
				float[] rmoveTo = TraceCircle(moveTarget, 150, 90)
				rider.SetPosition(rmoveTo[0],rmoveTo[1],rmoveTo[2])
				rider.SetAngle(moveTarget.GetAngleX(), moveTarget.GetAngleY(), moveTarget.GetAngleZ())
			endif
			count = count + 1
		endif
		i = i + 1
	endWhile

endFunction

Bool Function IsStandard(ObjectReference theHorse)
	Bool ret = false
	ReferenceAlias[] standard = new ReferenceAlias[5]
	standard[0] = WhiterunHorse
	standard[1] = WindhelmHorse
	standard[2] = RiftenHorse
	standard[3] = MarkarthHorse
	standard[4] = SolitudeHorse
	Int i = 0
	while (i < standard.length)
		if (standard[i].GetActorRef() && standard[i].GetActorRef() == theHorse)
			ret = true
			i = standard.length
		endIf
		i += 1
	endWhile
	return ret
EndFunction

ReferenceAlias[] Function GetAllTweakHorses(Bool excludeTweakPCHorseFaction = FALSE, Bool limitToCurrentHorseFaction = FALSE, Bool onlyHorsesNearPC = FALSE, Bool onlyHorsesFarAwayFromPC = FALSE)

	ReferenceAlias[] all = new ReferenceAlias[5]
	all[0] = WhiterunHorse
	all[1] = WindhelmHorse
	all[2] = RiftenHorse
	all[3] = MarkarthHorse
	all[4] = SolitudeHorse

	ReferenceAlias[] ret = new ReferenceAlias[6]
	int count = 0

	Int i = 0
	while (i != all.length && count != 5)
		; PlayerHorseFaction limits returned horses to ones that you OWN
		if (all[i].GetActorRef() && all[i].GetActorRef().IsInFaction(PlayerHorseFaction) && !all[i].GetActorRef().IsDead())
			if (!excludeTweakPCHorseFaction || !all[i].GetActorRef().IsInFaction(TweakPCHorseFaction))
				if (!limitToCurrentHorseFaction || all[i].GetActorRef().IsInFaction(TweakCurrentHorseFaction))
					if (!onlyHorsesNearPC || Game.GetPlayer().GetDistance(all[i].GetActorRef()) < 10000)
						if (!onlyHorsesFarAwayFromPC || Game.GetPlayer().GetDistance(all[i].GetActorRef()) > 10000)
						    ret[count] = all[i]
							count += 1
						endif
					endif
				endif
			endif
		endif
		i = i + 1
	endWhile

	; We allow 1 unowned horse (maybe summoned or borrowed)
	if (!ExtraHorse)
		ExtraHorse = (self As Quest).GetAlias(41) As ReferenceAlias
	endif
	if (ExtraHorse && ExtraHorse.GetActorRef() && !ExtraHorse.GetActorRef().IsDead())
		if (!excludeTweakPCHorseFaction || !ExtraHorse.GetActorRef().IsInFaction(TweakPCHorseFaction))
			if (!limitToCurrentHorseFaction || ExtraHorse.GetActorRef().IsInFaction(TweakCurrentHorseFaction))
				if (!onlyHorsesNearPC || Game.GetPlayer().GetDistance(ExtraHorse.GetActorRef()) < 10000)
					if (!onlyHorsesFarAwayFromPC || Game.GetPlayer().GetDistance(ExtraHorse.GetActorRef()) > 10000)
						ret[count] = ExtraHorse
						count += 1
					endif
				endif
			endif
		endif
	endif


	ReferenceAlias[] trim;
	if (count == 6)
		trim = new ReferenceAlias[6]
	elseif (count == 5)
		trim = new ReferenceAlias[5]
	elseif (count == 4)
		trim = new ReferenceAlias[4]
	elseif (count == 3)
		trim = new ReferenceAlias[3]		
	elseif (count == 2)
		trim = new ReferenceAlias[2]
	elseif (count == 1)
		trim = new ReferenceAlias[1]
	endif
	
	i = 0
	while (i < count)
		trim[i] = ret[i]
		i += 1
	endwhile
	
	return trim
	
endFunction

Float[] Function TraceCircle(Actor npc, Float radius = 500.0, Float angleOffset = 0.0)

    float a = npc.GetAngleZ();
    a = 450 - a
    if (a >= 360)
        a = a - 360
    endif 
    a  = a + angleOffset;
    if (a < 0) 
        a = a + 360
    endif
    if (a > 360)
        a = a - 360
    endif 

    Float xoffset = radius * Math.cos(a)
    Float yoffset = radius * Math.sin(a)

    Float[] ret = new Float[3]
    ret[0] =  (npc.GetPositionX() + xoffset)
    ret[1] =  (npc.GetPositionY() + yoffset)
    ret[2] = npc.GetPositionZ()
    return ret

endFunction